/**********************************************************************
 * $Id: dmt-ux.h 177 2006-09-24 23:00:54Z timo $
 *
 * DMT-UX
 * DSL Modem Tool for Unix
 *
 * written by Timo Boettcher
 * inspired by A. Matthoefer
 * with help of Michael Beckers
 *
 * (C) 2006 Timo Boettcher
 * This is free software. You may redistribute copies of it under the
 * terms of the GNU General Public License (Version 2)
 * <http://www.gnu.org/licenses/gpl.html>.
 * There is NO WARRANTY, to the extent permitted by law.
 *
 *********************************************************************/


#ifdef graphic
#define GRAPH_WIDTH_DMT 497
#define GRAPH_WIDTH_DMTUX 513
#define GRAPH_HEIGHT 61

#define MEDIUMGRAPH_WIDTH_DMT 532
#define MEDIUMGRAPH_WIDTH_DMTUX 548
#define MEDIUMGRAPH_HEIGHT 82

#define BIGGRAPH_WIDTH_DMT 532
#define BIGGRAPH_WIDTH_DMTUX 548
#define BIGGRAPH_HEIGHT 226

#define BIGGRAPH_GRAPH_LEFT 17
#define BIGGRAPH_GRAPH1_TOP 10
#define BIGGRAPH_GRAPH2_TOP 82
#define BIGGRAPH_GRAPH3_TOP 154
#define BIGGRAPH_GRAPH4_TOP 226
#define BIGGRAPH_GRAPH5_TOP 298

#define BIGGRAPH_HEAD 112

#define GRAPH_BITALLOC_GRID_WIDTH 16
#define GRAPH_BITALLOC_GRID_HEIGHT 4
#define GRAPH_NOISE_GRID_WIDTH 16
#define GRAPH_NOISE_GRID_HEIGHT 10
#define GRAPH_CHANCHARLOG_GRID_WIDTH 16
#define GRAPH_CHANCHARLOG_GRID_HEIGHT 10
#define GRAPH_ERRORSECS_GRID_WIDTH 20 
#define GRAPH_ERRORSECS_GRID_HEIGHT 10
#define GRAPH_SNR_GRID_WIDTH 20 
#define GRAPH_SNR_GRID_HEIGHT 10

#define TONE_KHZ_SCALEFACTOR 4.3125
#define BITALLOC_SCALEFACTOR 0.25
#define BITALLOC_FACTOR 4
#define NOISE_SCALEFACTOR 1
#define SNR_FACTOR 1
#define NOISEMARGIN_FACTOR 1
#define CHANCHARLOG_SCALEFACTOR 2
#define CHANCHARLOG_FACTOR 0.5
#define ERRORSECS_SCALEFACTOR 1
#define ERRORSECS_FACTOR 1
#define ERRORSECS_STRECH_FACTOR 10
#define SNR_SCALEFACTOR 1
#define SNR_FACTOR 1
#define SNR_STRECH_FACTOR 10
#endif //graphic

#define DROP_TONES_DMT 16
#define DROP_TONES_DMTUX 0 




enum {
#ifdef graphic
  COMMANDLINEOPTION_FONT,
  COMMANDLINEOPTION_IMAGEFILE,
  COMMANDLINEOPTION_IMAGEFILE_NOISE,
  COMMANDLINEOPTION_IMAGEFILE_BITALLOC,
  COMMANDLINEOPTION_IMAGEFILE_CHANCHARLOG,
#ifdef rrd
  COMMANDLINEOPTION_IMAGEFILE_RRD_ERRORSECS,
  COMMANDLINEOPTION_IMAGEFILE_RRD_SNR,
#endif //rrd
#endif //graphic
#ifdef rrd
  COMMANDLINEOPTION_RRDFILE_ERRORSECS,
  COMMANDLINEOPTION_RRDFILE_SNR,
#endif //rrd
  COMMANDLINEOPTION_PIDFILE,
  COMMANDLINEOPTION_DMTCOMPATIBLE
};

static struct option const long_options[] = {
  {"daemon", optional_argument, NULL, 'd'},
  {"dmt-compatible", no_argument, NULL, COMMANDLINEOPTION_DMTCOMPATIBLE},
  {"help", no_argument, NULL, 'h'},
  {"hostname", required_argument, NULL, 'H'},
#ifdef graphic
  {"font", required_argument, NULL, COMMANDLINEOPTION_FONT},
  {"imagefile", required_argument, NULL, 'I'},
  {"image-noise-file", required_argument, NULL, COMMANDLINEOPTION_IMAGEFILE_NOISE},
  {"image-bitalloc-file", required_argument, NULL, COMMANDLINEOPTION_IMAGEFILE_BITALLOC},
  {"image-chancharlog-file", required_argument, NULL, COMMANDLINEOPTION_IMAGEFILE_CHANCHARLOG},
#ifdef rrd
  {"image-rrd-errorsecs-file", required_argument, NULL, COMMANDLINEOPTION_IMAGEFILE_RRD_ERRORSECS},
  {"image-rrd-snr-file", required_argument, NULL, COMMANDLINEOPTION_IMAGEFILE_RRD_SNR},
#endif //rrd
#endif //graphic
  {"information", no_argument, NULL, 'i'},
#ifdef graphic
  {"legend", no_argument, NULL, 'l'},
#endif //graphic
  {"logfile", required_argument, NULL, 'L'},
  {"outputfile", required_argument, NULL, 'o'},
  {"password", required_argument, NULL, 'p'},
  {"port", required_argument, NULL, 'P'},
  {"pidfile", required_argument, NULL, COMMANDLINEOPTION_PIDFILE},
#ifdef rrd
  {"rrd-errorsecs-file", required_argument, NULL, COMMANDLINEOPTION_RRDFILE_ERRORSECS},
  {"rrd-snr-file", required_argument, NULL, COMMANDLINEOPTION_RRDFILE_SNR},
#endif //rrd
#ifdef graphic
  {"transparency", no_argument, NULL, 't'},
#endif //graphic
  {"username", required_argument, NULL, 'u'},
  {"version", no_argument, NULL, 'v'},
  {NULL, 0, NULL, 0}
};






typedef struct config_t{
	char* hostname;
	char* port;
	char* username;
	char* password;
	char* outputfile;
	unsigned int checksync;
	unsigned int daemon;
	unsigned int information;
	char* pidfile;
	char* logfile;
#ifdef graphic
	char* imagefile;
	char* imagefile_bitalloc;
	char* imagefile_noise;
	char* imagefile_chancharlog;
	char* imagefile_rrd_errorsecs;
	char* imagefile_rrd_snr;
	char* font;
	unsigned int legend;
	unsigned int transparency;
#endif //graphic
#ifdef rrd
	char* rrdfile_errorsecs;
	char* rrdfile_snr;
#endif //rrd
} Config;




void mainloop();
void init_dmt(int argc, char** argv);
int read_args(int argc, char** argv);
void usage();
void init_dmt_data(int sock);
int connect_to_modem();
void cleanup_dmt();
void handle_socket(int csock);
void send_socket(int csock, char* data);

void information_prepare();
void information_print();
void information_kill();
void information_init();
char* linelength();
unsigned int get_bitalloc_upstream_downstream(IntArray bitalloc);

#ifdef graphic
void graphs_write_all(char* imagefile);
void graphs_write_bitalloc(char* imagefile);
void graphs_create_bitalloc(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text);
void graphs_write_noise(char* imagefile);
void graphs_create_noise(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text);
void graphs_write_chancharlog(char* imagefile);
void graphs_create_chancharlog(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text);
#ifdef rrd
void graphs_write_rrd_errorsecs(char* imagefile);
void graphs_create_rrd_errorsecs(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text);
void graphs_write_rrd_snr(char* imagefile);
void graphs_create_rrd_snr(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text);
#endif //rrd
void graphs_create_refscale(Image* graph, unsigned int x_start, unsigned int width, unsigned int y, int step, unsigned int pos_first_value, double scalefactor, char* unit);
void graphs_create_valuescale(Image* graph, unsigned int x, unsigned int y_start, unsigned int height, unsigned int step, unsigned int pos_first_value, double scalefactor);
#endif //graphic

