/**********************************************************************
 * $Id: graphic.h 177 2006-09-24 23:00:54Z timo $
 *
 * DMT-UX
 * DSL Modem Tool for Unix
 *
 * written by Timo Boettcher
 * inspired by A. Matthoefer
 * with help of Michael Beckers
 *
 * (C) 2006 Timo Boettcher
 * This is free software. You may redistribute copies of it under the
 * terms of the GNU General Public License (Version 2)
 * <http://www.gnu.org/licenses/gpl.html>.
 * There is NO WARRANTY, to the extent permitted by law.
 *
 *********************************************************************/

#ifndef _GRAPHIC_H
#define _GRAPHIC_H
typedef struct image_t{
	unsigned char* pixels;
	unsigned int width;
	unsigned int height;
	unsigned int bpp;
} Image;

typedef struct xpm_colors_t{
	char* character;
	unsigned char* color;
} XpmColors;

typedef unsigned char* Color;

void image_init(Image* image, unsigned int width, unsigned int height, unsigned int bpp);
void image_kill(Image* image);
void image_paint_pixel(Image* image, unsigned int x, unsigned int y, Color color);
void image_paint_pixel_alpha(Image* image, unsigned int x, unsigned int y, Color color, Color alpha);
void image_paint_alpha(Image* image, unsigned int x, unsigned int y, Color alpha);
void image_clear(Image* image, Color color_bg);
void image_clear_alpha(Image* image, Color color_bg, Color alpha);
void image_paint_rect(Image* image, unsigned int x_start, unsigned int y_start, unsigned int x_end, unsigned int y_end, Color color_bg);
void image_paint_hor_line(Image* image, unsigned int x1, unsigned int x2, unsigned int y, Color color);
void image_paint_vert_line(Image* image, unsigned int x, unsigned int y1, unsigned int y2, Color color);
void image_paint_grid(Image* image, unsigned int xspacing, unsigned int yspacing, Color color);
void image_paint_grid2(Image* image, unsigned int x_start, unsigned int y_start, unsigned int x_spacing, unsigned int y_spacing, unsigned int x_end, unsigned int y_end, Color color);
void image_paint_xpm(Image* image, char** xpm, unsigned int x_start, unsigned int y_start);
void image_paint_text(Image* image, unsigned int pos_x, unsigned int pos_y, int angle, unsigned int size_x, unsigned int size_y, char* font, Color color, char* text);
void image_paint_textbitmap(Image* image, FT_Bitmap* bitmap, unsigned int x, unsigned int y, Color color);
void image_save_png(Image* image, char* filename);

#endif
