/**********************************************************************
 * $Id: helper.c 177 2006-09-24 23:00:54Z timo $
 *
 * DMT-UX
 * DSL Modem Tool for Unix
 *
 * written by Timo Boettcher
 * inspired by A. Matthoefer
 * with help of Michael Beckers
 *
 * (C) 2006 Timo Boettcher
 * This is free software. You may redistribute copies of it under the
 * terms of the GNU General Public License (Version 2)
 * <http://www.gnu.org/licenses/gpl.html>.
 * There is NO WARRANTY, to the extent permitted by law.
 *
 *********************************************************************/

#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>

#include "helper.h"

int logtofile=0;
FILE* logfile;

/*****************************************
 * String Functions
 ****************************************/


int value_cap(int value, int min, int max) {
	if (value>max) {
		value=max;
	}
	if (value<min) {
		value=min;
	}
	return value;
}

void tr(char* source) {
	if (source) {
		if (strlen(source)>1) {
			char* temp = dmalloc(sizeof(char)*(strlen(source)+1));
			strcpy(temp,source);
			char* tempcopy = temp;
			while (tempcopy[0]==' ') {
				tempcopy=tempcopy+1;
			}
			while (tempcopy[strlen(tempcopy)-1]==' ') {
				tempcopy[strlen(tempcopy)-1]='\0';
			}
			strcpy(source,tempcopy);
			free(temp);
		}
	}
}

char* get_token_value(char* token, char divide) {
	char* datastartpos;
	char* dest;

	datastartpos = strrchr(token,divide) + 1;
	if (datastartpos) {
		dest = dmalloc((strlen(datastartpos)+1)*sizeof(char));
		strcpy(dest, datastartpos);
	} else {
		dest = dmalloc(1*sizeof(char));
		message_write(MSGLEVEL_WARNING,"Token found, but no data delimiter. Skipping.\n");
	}
	return dest;
}

char* get_firstvalue(char* source) {
	char* result;
	char* nexttoken;
	nexttoken=strtok(source, " ");
	if (nexttoken) {
		result=dmalloc(sizeof(char)*(strlen(nexttoken)+1));
		strcpy(result, nexttoken);
		return result;
	} else {
		nexttoken=strtok(NULL, " ");
		if (nexttoken) {
			result=dmalloc(sizeof(char)*(strlen(nexttoken)+1));
			strcpy(result, nexttoken);
			return result;
		} else {
			return source;
		}
	}
}

char* get_secondvalue(char* source) {
	char* result;
	char* nexttoken;
	nexttoken=strtok(NULL, " ");
	if (nexttoken) {
		result=dmalloc(sizeof(char)*(strlen(nexttoken)+1));
		strcpy(result, nexttoken);
		return result;
	} else {
		return source;
	}
}

void* dmalloc(unsigned int size) {
	char* pointer = malloc(size);
	if (pointer != NULL) {
		memset (pointer, 0, size);
	} else {
		fprintf(stderr, "malloc %d failed\n", size);
		exit(1);
	}
	return pointer;
}


void message_write(int msglevel, char* message) {
	if (logtofile) {
		if (!logfile) {
			fprintf(stderr, "could not write to logfile\n");
			logtofile=0;
		} else {
			fprintf(logfile,message);
			fflush(logfile);
		}
	} else {
		fprintf(stderr, message);
	}
}


void message_log_open(char* logfilename) {
	struct stat statbuf;
	if (stat(logfilename, &statbuf) < 0) {
		fprintf(stderr, "Couldn't stat Logfile \"%s\". Creating it.\n", logfilename);
	} else {
		if (S_ISREG(statbuf.st_mode) == 0) {
			fprintf(stderr, "Logfile is not a file \"%s\"\n", logfilename);
		}
	}
        logfile = fopen(logfilename, "a");
	if (logfile) {
		logtofile=1;
	}
}

void message_log_close() {
	if (logfile) {
		fclose(logfile);
		logtofile=0;
	}
}

void pidfile_write(char* pidfilename, pid_t pid) {
	struct stat statbuf;
	FILE* pidfile;
	if (stat(pidfilename, &statbuf) >= 0) {
		if (S_ISREG(statbuf.st_mode) == 0) {
			fprintf(stderr, "Pidfile is not a file \"%s\"\n", pidfilename);
		}
	}
        pidfile = fopen(pidfilename, "w");
	if (pidfile) {
		fprintf(pidfile,"%i\n",(int)pid);
        	fclose(pidfile);
	}
}

