/**********************************************************************
 * $Id: speedtouch.c 177 2006-09-24 23:00:54Z timo $
 *
 * DMT-UX
 * DSL Modem Tool for Unix
 *
 * written by Timo Boettcher
 * inspired by A. Matthoefer
 * with help of Michael Beckers
 *
 * (C) 2006 Timo Boettcher
 * This is free software. You may redistribute copies of it under the
 * terms of the GNU General Public License (Version 2)
 * <http://www.gnu.org/licenses/gpl.html>.
 * There is NO WARRANTY, to the extent permitted by law.
 *
 *********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "../types.h"
#include "../helper.h"
#include "speedtouch.h"
int command;
char* username;
char* password;
IntArray* dsl_bitalloc;
IntArray* dsl_snr;
IntArray* dsl_noisemargin;
IntArray* dsl_chancharlog;
unsigned int* dsl_errorsecs;
Modem* modemdata;
StringList* commandlist;
void (*callback_sendsocket)(int, char*);
int sock;
void (*callback_messagewrite)(int MESSAGE_LEVEL, char* message);



void parser_init(char* ausername, char* apassword, IntArray* adsl_bitalloc, IntArray* adsl_snr, IntArray* adsl_noisemargin, IntArray* adsl_chancharlog, StringList* acommandlist, unsigned int* adsl_errorsecs, Modem* amodemdata, void (*acallback_sendsocket)(int, char*), int asock, void (*acallback_messagewrite)(int MESSAGE_LEVEL, char* message)) {
	username=ausername;
	password=apassword;

	dsl_bitalloc=adsl_bitalloc;
	dsl_snr=adsl_snr;
	dsl_noisemargin=adsl_noisemargin;
	dsl_chancharlog=adsl_chancharlog;
	dsl_errorsecs=adsl_errorsecs;
	modemdata=amodemdata;

	command=9;
	commandlist=acommandlist;
	callback_sendsocket=acallback_sendsocket;
	sock=asock;
	callback_messagewrite=acallback_messagewrite;
}




unsigned char handle_data(char* data) {
	char* handled_data = malloc(sizeof(char) * (strlen(data)+1));
	strcpy(handled_data,data);

	char* promptpos;

	char* promptusername= "Username";
	char* promptpassword= "Password";

	promptpos = strstr(handled_data, promptusername);
	if (promptpos) {
		message_write(MSGLEVEL_WARNING, "Got asked for username\n");
		char* uname = malloc(sizeof(char) * (3 + strlen(username)));
		sprintf(uname, "%s\r\n", username);
		callback_sendsocket(sock, uname);
		free(uname);
		free(handled_data);
		return 1;
	}
	promptpos = strstr(handled_data, promptpassword);
	if (promptpos) {
		message_write(MSGLEVEL_WARNING, "Got asked for password\n");
		char* pass = malloc(sizeof(char) * (3 + strlen(password)));
		sprintf(pass, "%s\r\n", password);
		callback_sendsocket(sock, pass);
		free(pass);
		free(handled_data);
		return 2;
	}

	char* prompt = malloc(sizeof(char) * (5 + strlen(username)));
	sprintf(prompt, "{%s}=>", username);


	promptpos = strstr(handled_data, prompt);
	free(prompt);
	free(handled_data);
	if (promptpos) {
		command++;
		switch(command) {
		case 10:
			message_write(MSGLEVEL_WARNING, "Fetching Environment\n");
			callback_sendsocket(sock, "env list\r\n");
			return command;
		case 11:
			message_write(MSGLEVEL_WARNING, "Fetching debug stats\n");
			callback_sendsocket(sock, "system debug stats\r\n");
			return command;
		case 12:
			message_write(MSGLEVEL_WARNING, "Fetching adsl info\n");
			callback_sendsocket(sock, "adsl info\r\n");
			return command;
		case 13:
			message_write(MSGLEVEL_WARNING, "Fetching adsl data\n");
			callback_sendsocket(sock, "debug exec cmd=\'tdsl getData all\'\r\n");
			return command;
		case 14:
			message_write(MSGLEVEL_WARNING, "Closing connection\n");
			callback_sendsocket(sock, "exit\r\n");
			return command;
		}
		message_write(MSGLEVEL_WARNING, "Got a prompt, but didn't sent a command\n");
	}
	return 0;
}


void parse_data(StringList* list) {
	StringListItem* current = list->first;
	char* nexttoken;
	char* datastartpos;
	int i;

	while (current) {
		if (current->answer) {
			nexttoken=strtok(current->answer,"\n\r");
			while (nexttoken) {
				//printf("C=%i\n",current->command);
				//printf("T=%s\n",nexttoken);
				switch(current->command) {
				case 10:
					//env list
					for (;;) {
						if (strstr(nexttoken, "_COMPANY_NAME=")) {
							modemdata->modemvendor = get_token_value(nexttoken, '=');
							break;
						}
						if (strstr(nexttoken, "_PROD_NAME=")) {
							modemdata->modemname = get_token_value(nexttoken, '=');
							break;
						}
						if (strstr(nexttoken, "_PROD_NUMBER=")) {
							modemdata->modemnumber = get_token_value(nexttoken, '=');
							break;
						}
						if (strstr(nexttoken, "_BOARD_SERIAL_NBR=")) {
							modemdata->modemserial = get_token_value(nexttoken, '=');
							break;
						}
						if (strstr(nexttoken, "_BUILD=")) {
							modemdata->modemfirmware = get_token_value(nexttoken, '=');
							break;
						}
						if (strstr(nexttoken, "_BOOTLOADER_VERSION=")) {
							modemdata->modembootloader = get_token_value(nexttoken, '=');
							break;
						}
						if (strstr(nexttoken, "_BOARD_NAME=")) {
							modemdata->modemboardname = get_token_value(nexttoken, '=');
							break;
						}
						if (strstr(nexttoken, "_USB_MACADDR=")) {
							break;
						}
						if (strstr(nexttoken, "_MACADDR=")) {
							modemdata->modemmacaddr = get_token_value(nexttoken, '=');
							break;
						}
						break;
					}
					break;
				case 11:
					//system debug stats
					break;
				case 12:
					//adsl info
					for (;;) {
						if (strstr(nexttoken, "Modemstate")) {
							modemdata->modemstate = get_token_value(nexttoken, ':');
							break;
						}
						if (strstr(nexttoken, "Operation Mode")) {
							modemdata->modemoperationmode = get_token_value(nexttoken, ':');
							break;
						}
						if (strstr(nexttoken, "Channel Mode")) {
							modemdata->modemchannelmode = get_token_value(nexttoken, ':');
							break;
						}
						if (strstr(nexttoken, "Margin")) {
							modemdata->modemmargin = get_token_value(nexttoken, ':');
							break;
						}
						if (strstr(nexttoken, "Attenuation")) {
							modemdata->modemattenuation = get_token_value(nexttoken, ':');
							break;
						}
						if (strstr(nexttoken, "OutputPower")) {
							modemdata->modemoutputpower = get_token_value(nexttoken, ':');
							break;
						}
						if (strstr(nexttoken, "VendorSpecific")) {
							modemdata->vendorspecific = get_token_value(nexttoken, ':');
							break;
						}
						if (strstr(nexttoken, "Vendor")) {
							nexttoken=strtok(NULL,"\n\r");
							nexttoken=strtok(NULL,"\n\r");
							if (strstr(nexttoken, "Vendor")) {
								modemdata->vendor = get_token_value(nexttoken, ':');
							}
							break;
						}
						if (strstr(nexttoken, "StandardRevisionNr")) {
							modemdata->revision = get_token_value(nexttoken, ':');
							break;
						}
						if (strstr(nexttoken, "Available Bandwidth")) {
							for(i=0; i<2; i++) {
								nexttoken=strtok(NULL,"\n\r");
								if (nexttoken) {
									if (strstr(nexttoken, "Downstream")) {
										modemdata->downstream = get_token_value(nexttoken, ':');
									}
									if (strstr(nexttoken, "Upstream")) {
										modemdata->upstream = get_token_value(nexttoken, ':');
									}
								} else {
									message_write(MSGLEVEL_WARNING,"Expected Downstream and Upstream Information after \"Available Bandwidth\" line. Skipping.\n");
								}
							}
							break;
						}
						if (strstr(nexttoken, "Transfer statistics")) {
							for (;;) {
								nexttoken=strtok(NULL,"\n\r");
								if (!nexttoken) {
									break;
								}
								if (strstr(nexttoken, "Near end failures current day")) {
									for (;;) {
										nexttoken=strtok(NULL,"\n\r");
										if (!nexttoken) {
											break;
										}
										if (strstr(nexttoken, "Errored seconds:")) {
											datastartpos = strrchr(nexttoken,':') + 1;
											if (datastartpos) {
												char* errorsecinfo = malloc((strlen(datastartpos)+1)*sizeof(char));
												strcpy(errorsecinfo, datastartpos);

												datastartpos = strstr(errorsecinfo,"seconds");
												datastartpos[0]='\0';

												tr(errorsecinfo);
												*dsl_errorsecs=atoi(errorsecinfo);
												free(errorsecinfo);
											} else {
												message_write(MSGLEVEL_WARNING,"Token found, but no data delimiter. Skipping.\n");
											}
											break;
										}
									}
									break;
								}
							}
							
						}
						break;
					}
					break;
				case 13:
					//debug exec cmd=\'tdsl getData all\'
					for (;;) {
						if (strstr(nexttoken, "bitalloc start")) {
							parse_tonedata(dsl_bitalloc, "bitalloc end", nexttoken, 0);
							break;
						}
						if (strstr(nexttoken, "snr start")) {
							parse_tonedata(dsl_snr, "snr end", nexttoken, 1);
							break;
						}
						if (strstr(nexttoken, "noisemargin start")) {
							parse_tonedata(dsl_noisemargin, "noisemargin end", nexttoken, 0);
							break;
						}
						if (strstr(nexttoken, "chanCharLog start")) {
							parse_tonedata(dsl_chancharlog, "chanCharLog end", nexttoken, 0);
							break;
						}
						break;
					}

					break;
				case 14:
					//exit
					break;
				}
				
				nexttoken=strtok(NULL,"\n\r");
			}
			//printf(current->answer);
		}
		current = current->next;
	}
}



void parse_tonedata(IntArray* array, char* end_str, char* token, int factor) {
	token=strtok(NULL,"\n\r");
	int tone = 0;
	if (token) {
		while (!strstr(token, end_str)) {
			char* tonestr = malloc((strlen(token) + 1) * sizeof(char));
			strcpy(tonestr, token);
			tonestr[4] = '\0';
			if (strcmp(tonestr, "tone") != 0) {
				message_write(MSGLEVEL_WARNING,"no tone\n");
				free(tonestr);
				break;
			}
			token = strchr(token, ' ') + 1;
			char* tonenr = malloc((strlen(token) + 1) * sizeof(char));
			strcpy(tonenr, token);
			char* tonenrend = strchr(tonenr, ':');
			if (!tonenrend) {
				message_write(MSGLEVEL_WARNING,"no tonenumber\n");
				free(tonestr);
				free(tonenr);
				break;
			}
			tonenrend[0] = '\0';
			if (atoi(tonenr) != tone) {
				char* message = malloc(sizeof(char) * 1024);
				sprintf(message,"wrong tonenumber. expected %i, got %s\n", tone, tonenr);
				message_write(MSGLEVEL_WARNING,message);
				free(message);
				free(tonestr);
				free(tonenr);
				break;
			}
			token = strchr(token, ' ');
			do{
				token++;
				
				char* tonedata = malloc(sizeof(char) * (strlen(token)+1));
				strcpy(tonedata, token);
				char* tonedata_end = strchr(tonedata, ' ');
				if (!tonedata_end && tonedata[0] == '\0') {
					free(tonedata);
					break;
				}
				if (tonedata_end) {
					tonedata_end[0]='\0';
				}

				if (factor<0) {
					intarray_add(array, atoi(tonedata));
				} else if (factor>0) {
					if (atoi(tonedata)>0) {
						intarray_add(array, atoi(tonedata));
					} else {
						intarray_add(array, 0);
					}
				} else { //factor ==0
					intarray_add(array, abs(atoi(tonedata)));
				}
				free(tonedata);

				tone++;
				token = strchr(token, ' ');
			}while (token != NULL);
			token=strtok(NULL,"\n\r");
			free(tonestr);
			free(tonenr);
		}
	}
}

