/**********************************************************************
 * $Id: rrdstat.c 168 2006-09-19 21:56:29Z timo $
 *
 * DMT-UX
 * DSL Modem Tool for Unix
 *
 * written by Timo Boettcher
 * inspired by A. Matthoefer
 * with help of Michael Beckers
 *
 * (C) 2006 Timo Boettcher
 * This is free software. You may redistribute copies of it under the
 * terms of the GNU General Public License (Version 2)
 * <http://www.gnu.org/licenses/gpl.html>.
 * There is NO WARRANTY, to the extent permitted by law.
 *
 *********************************************************************/

#ifdef rrd

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <sys/time.h>

#include <rrd.h>

#include "helper.h"
#include "rrdstat.h"

void rrdstat_create_errorsecs(char* rrdfilename) {
	char *rrdargv[18] ;
	int result;

	rrdargv[0]="create";
	rrdargv[1]=rrdfilename;
	rrdargv[2]="--step";
	rrdargv[3]="300";			//store one value every 5 minutes
	rrdargv[4]="DS:errorsecs:GAUGE:900:0:100";
	rrdargv[5]="RRA:MIN:0:1:144";		//store each value for a day
	rrdargv[6]="RRA:AVERAGE:0:1:144";
	rrdargv[7]="RRA:MAX:0:1:144";
	rrdargv[8]="RRA:MIN:0:6:168";		//store the hours average for a week
	rrdargv[9]="RRA:AVERAGE:0:6:168";
	rrdargv[10]="RRA:MAX:0:6:168";
	rrdargv[11]="RRA:MIN:0:36:180";		//store six hours average for a month
	rrdargv[12]="RRA:AVERAGE:0:36:180";
	rrdargv[13]="RRA:MAX:0:36:180";
	rrdargv[14]="RRA:MIN:0:144:365";	//store the days average for a year
	rrdargv[15]="RRA:AVERAGE:0:144:365";
	rrdargv[16]="RRA:MAX:0:144:365";
	rrdargv[17]=NULL;


	result = rrd_create(17,rrdargv);
	if (result != 0) {
		if (rrd_test_error()) {
			char* message = dmalloc(sizeof(char) * 1024);
			sprintf(message, "rrd_create: %s\n", rrd_get_error()); 
			message_write(MSGLEVEL_WARNING,message);
			free(message);
		}
	}
	rrd_clear_error();
}

void rrdstat_create_snr(char* rrdfilename) {
	char *rrdargv[20] ;
	int result;

	rrdargv[0]="create";
	rrdargv[1]=rrdfilename;
	rrdargv[2]="--step";
	rrdargv[3]="300";			//store one value every 5 minutes
	rrdargv[4]="DS:snr_low:GAUGE:900:0:100";
	rrdargv[5]="DS:snr_LQ:GAUGE:900:0:100";
	rrdargv[6]="DS:snr_high:GAUGE:900:0:100";
	rrdargv[7]="RRA:MIN:0:1:144";		//store each value for a day
	rrdargv[8]="RRA:AVERAGE:0:1:144";
	rrdargv[9]="RRA:MAX:0:1:144";
	rrdargv[10]="RRA:MIN:0:6:168";		//store the hours average for a week
	rrdargv[11]="RRA:AVERAGE:0:6:168";
	rrdargv[12]="RRA:MAX:0:6:168";
	rrdargv[13]="RRA:MIN:0:36:180";		//store six hours average for a month
	rrdargv[14]="RRA:AVERAGE:0:36:180";
	rrdargv[15]="RRA:MAX:0:36:180";
	rrdargv[16]="RRA:MIN:0:144:365";	//store the days average for a year
	rrdargv[17]="RRA:AVERAGE:0:144:365";
	rrdargv[18]="RRA:MAX:0:144:365";
	rrdargv[19]=NULL;


	result = rrd_create(19,rrdargv);
	if (result != 0) {
		if (rrd_test_error()) {
			char* message = dmalloc(sizeof(char) * 1024);
			sprintf(message, "rrd_create: %s\n", rrd_get_error()); 
			message_write(MSGLEVEL_WARNING,message);
			free(message);
		}
	}
	rrd_clear_error();
}

void rrdstat_update_errorsecs(char* rrdfilename, struct timeval tp, int value) {
	char *rrdargv[4];
	char* text;
	text=malloc(256);
	int result;
	
	rrdargv[0]="update";
	rrdargv[1]=rrdfilename;
	sprintf(text,"%u:%i",(long)(tp.tv_sec),value);
	rrdargv[2]=text;
	rrdargv[3]=NULL;


	result = rrd_update(3,rrdargv);
	if (result != 0) {
		if (rrd_test_error()) {
			char* message = dmalloc(sizeof(char) * 1024);
			sprintf(message, "rrd_update: %s\n", rrd_get_error()); 
			message_write(MSGLEVEL_WARNING,message);
			free(message);
		}
	}
	rrd_clear_error();
	free(text);
}
void rrdstat_update_snr(char* rrdfilename, struct timeval tp, int value_low, int value_LQ, int value_high) {
	char *rrdargv[4];
	char* text;
	text=malloc(256);
	int result;
	
	rrdargv[0]="update";
	rrdargv[1]=rrdfilename;
	sprintf(text,"%u:%i:%i:%i",(long)(tp.tv_sec),value_low,value_LQ,value_high);
	rrdargv[2]=text;
	rrdargv[3]=NULL;


	result = rrd_update(3,rrdargv);
	if (result != 0) {
		if (rrd_test_error()) {
			char* message = dmalloc(sizeof(char) * 1024);
			sprintf(message, "rrd_update: %s\n", rrd_get_error()); 
			message_write(MSGLEVEL_WARNING,message);
			free(message);
		}
	}
	rrd_clear_error();
	free(text);
}

void rrdstat_rrdgraph_errorsecs(char* rrdfilename, char* imagefilename) {
	char *rrdargv[15] ;
	int i;
	char* text;

	int result;

	char **calcpr  = NULL;
	int xsize, ysize;
	double ymin, ymax;

	text=malloc(sizeof(char)*(38+strlen(rrdfilename)));
	xsize=600;
	ysize=480;

	rrdargv[0]="graph";
	rrdargv[1]=imagefilename;
	rrdargv[2]="-a";
	rrdargv[3]="PNG";
	rrdargv[4]="--start";
	rrdargv[5]="end-24h";
	rrdargv[6]="--end";
	rrdargv[7]="now";
	rrdargv[8]="--step";
	rrdargv[9]="300";
	rrdargv[10]="--title";
	rrdargv[11]="Test Graph";
	sprintf(text,"DEF:myerrorsecs=%s:errorsecs:AVERAGE:start=end-1h",rrdfilename);
	rrdargv[12]=text;
	rrdargv[13]="LINE1:myerrorsecs#00CC00:Errored Seconds";
	rrdargv[14]=NULL;

	rrd_clear_error();
	result = rrd_graph(14, rrdargv, &calcpr, &xsize, &ysize, NULL, &ymin, &ymax);

	if (result != 0) {
		if (rrd_test_error()) {
			if (calcpr) {
				for (i=0; (calcpr[i]); i++) free(calcpr[i]);
					calcpr = NULL;
			}
			char* message = dmalloc(sizeof(char) * 1024);
			sprintf(message, "rrd_graph: %s\n", rrd_get_error()); 
			message_write(MSGLEVEL_WARNING,message);
			free(message);
		}
	}
	rrd_clear_error();
	free(text);
}

void rrdstat_rrdgraph_snr(char* rrdfilename, char* imagefilename) {
	char *rrdargv[19];
	int i;
	char* text;

	int result;

	char **calcpr  = NULL;
	int xsize, ysize;
	double ymin, ymax;

	text=malloc(sizeof(char)*(38+strlen(rrdfilename)));
	xsize=600;
	ysize=480;

	rrdargv[0]="graph";
	rrdargv[1]=imagefilename;
	rrdargv[2]="-a";
	rrdargv[3]="PNG";
	rrdargv[4]="--start";
	rrdargv[5]="end-24h";
	rrdargv[6]="--end";
	rrdargv[7]="now";
	rrdargv[8]="--step";
	rrdargv[9]="300";
	rrdargv[10]="--title";
	rrdargv[11]="Test Graph";
	sprintf(text,"DEF:mysnr_low=%s:snr_low:AVERAGE:start=end-1h",rrdfilename);
	rrdargv[12]=text;
	sprintf(text,"DEF:mysnr_LQ=%s:snr_LQ:AVERAGE:start=end-1h",rrdfilename);
	rrdargv[13]=text;
	sprintf(text,"DEF:mysnr_high=%s:snr_high:AVERAGE:start=end-1h",rrdfilename);
	rrdargv[14]=text;
	rrdargv[15]="LINE1:mysnr_low#00CC00:minimum Signal to Noise Ratio";
	rrdargv[16]="LINE2:mysnr_LQ#CCCC00:average Signal to Noise Ratio";
	rrdargv[17]="LINE3:mysnr_high#CC0000:maximum Signal to Noise Ratio";
	rrdargv[18]=NULL;

	rrd_clear_error();
	result = rrd_graph(18, rrdargv, &calcpr, &xsize, &ysize, NULL, &ymin, &ymax);

	if (result != 0) {
		if (rrd_test_error()) {
			if (calcpr) {
				for (i=0; (calcpr[i]); i++) free(calcpr[i]);
					calcpr = NULL;
			}
			char* message = dmalloc(sizeof(char) * 1024);
			sprintf(message, "rrd_graph: %s\n", rrd_get_error()); 
			message_write(MSGLEVEL_WARNING,message);
			free(message);
		}
	}
	rrd_clear_error();
	free(text);
}

void rrdstat_fetch_errorsecs(char* rrdfilename, rrd_value_t** rrddata, time_t* starttime, time_t* endtime, unsigned long* steptime, unsigned long* dscount, char*** dsnames) {
	char *rrdargv[10] ;
	int result;

	rrdargv[0]="fetch";
	rrdargv[1]=rrdfilename;
	rrdargv[2]="AVERAGE";
	rrdargv[3]="-r";
	rrdargv[4]="300";
	rrdargv[5]="-s";
	rrdargv[6]="end-24h";
	rrdargv[7]="-e";
	rrdargv[8]="now";
	rrdargv[9]=NULL;

	rrd_clear_error();
	result = rrd_fetch(9, rrdargv, starttime, endtime, steptime, dscount, dsnames, rrddata);
	if (result == -1) {
		char* message = dmalloc(sizeof(char) * 1024);
		sprintf(message, "Error while retrieving RRD dataset names: %s\n", rrd_get_error());
		message_write(MSGLEVEL_WARNING,message);
		free(message);
	}
}
void rrdstat_fetch_snr(char* rrdfilename, rrd_value_t** rrddata, time_t* starttime, time_t* endtime, unsigned long* steptime, unsigned long* dscount, char*** dsnames) {
	char *rrdargv[10] ;
	int result;

	rrdargv[0]="fetch";
	rrdargv[1]=rrdfilename;
	rrdargv[2]="AVERAGE";
	rrdargv[3]="-r";
	rrdargv[4]="300";
	rrdargv[5]="-s";
	rrdargv[6]="end-24h";
	rrdargv[7]="-e";
	rrdargv[8]="now";
	rrdargv[9]=NULL;

	rrd_clear_error();
	result = rrd_fetch(9, rrdargv, starttime, endtime, steptime, dscount, dsnames, rrddata);
	if (result == -1) {
		char* message = dmalloc(sizeof(char) * 1024);
		sprintf(message, "Error while retrieving RRD dataset names: %s\n", rrd_get_error());
		message_write(MSGLEVEL_WARNING,message);
		free(message);
	}
}

#endif //rrd
